@@ -19,6 +19,7 @@
 import offlineimap.version
 import re, time, sys, traceback, threading, thread
 from StringIO import StringIO
+from Queue import Empty
 
 debugtypes = {'imap': 'IMAP protocol debugging',
               'maildir': 'Maildir repository debugging',
@@ -151,17 +152,17 @@ class UIBase:
 
     ################################################## WARNINGS
     def msgtoreadonly(s, destfolder, uid, content, flags):
-        if not (config.has_option('general', 'ignore-readonly') and config.getboolean("general", "ignore-readonly")):
+        if not (s.config.has_option('general', 'ignore-readonly') and s.config.getboolean("general", "ignore-readonly")):
             s.warn("Attempted to synchronize message %d to folder %s[%s], but that folder is read-only.  The message will not be copied to that folder." % \
                    (uid, s.getnicename(destfolder), destfolder.getname()))
 
     def flagstoreadonly(s, destfolder, uidlist, flags):
-        if not (config.has_option('general', 'ignore-readonly') and config.getboolean("general", "ignore-readonly")):
+        if not (s.config.has_option('general', 'ignore-readonly') and s.config.getboolean("general", "ignore-readonly")):
             s.warn("Attempted to modify flags for messages %s in folder %s[%s], but that folder is read-only.  No flags have been modified for that message." % \
                    (str(uidlist), s.getnicename(destfolder), destfolder.getname()))
 
     def deletereadonly(s, destfolder, uidlist):
-        if not (config.has_option('general', 'ignore-readonly') and config.getboolean("general", "ignore-readonly")):
+        if not (s.config.has_option('general', 'ignore-readonly') and s.config.getboolean("general", "ignore-readonly")):
             s.warn("Attempted to delete messages %s in folder %s[%s], but that folder is read-only.  No messages have been deleted in that folder." % \
                    (str(uidlist), s.getnicename(destfolder), destfolder.getname()))
 
@@ -208,9 +209,15 @@ class UIBase:
                                              s.getnicename(srcrepos),
                                              s.getnicename(destrepos)))
 
-    def validityproblem(s, folder, saved, new):
-        s.warn("UID validity problem for folder %s (saved %d; got %d); skipping it" % \
-               (folder.getname(), saved, new))
+    def skippingfolder(s, folder):
+        """Called when a folder sync operation is started."""
+        if s.verbose >= 0:
+            s._msg("Skipping %s (not changed)" % folder.getname())
+
+    def validityproblem(s, folder):
+        s.warn("UID validity problem for folder %s (repo %s) (saved %d; got %d); skipping it" % \
+               (folder.getname(), folder.getrepository().getname(),
+                folder.getsaveduidvalidity(), folder.getuidvalidity()))
 
     def loadmessagelist(s, repos, folder):
         if s.verbose > 0:
@@ -316,9 +323,15 @@ class UIBase:
         s.delThreadDebugLog(thread)
         s.unregisterthread(thread)
 
+    ################################################## Hooks
+
+    def callhook(s, msg):
+        if s.verbose >= 0:
+            s._msg(msg)
+
     ################################################## Other
 
-    def sleep(s, sleepsecs):
+    def sleep(s, sleepsecs, siglistener):
         """This function does not actually output anything, but handles
         the overall sleep, dealing with updates as necessary.  It will,
         however, call sleeping() which DOES output something.
@@ -328,7 +341,12 @@ class UIBase:
 
         abortsleep = 0
         while sleepsecs > 0 and not abortsleep:
-            abortsleep = s.sleeping(1, sleepsecs)
+            try:
+                abortsleep = siglistener.get_nowait()
+                # retrieved signal while sleeping: 1 means immediately resynch, 2 means immediately die
+            except Empty:
+                # no signal
+                abortsleep = s.sleeping(1, sleepsecs)
             sleepsecs -= 1
         s.sleeping(0, 0)               # Done sleeping.
         return abortsleep
